#!/usr/bin/python
# -*- coding: utf-8 -*-
import os,sys
from yocto_api import *
from yocto_display import *
from yocto_relay import *
from yocto_anbutton import *

# some global state variables
lastfunction=""          # memorize last keypad key change
lastvalue=0;
currentcode="";        # code being entered
refreshneeded=True;      # display refresh control
keyboardDisabled=False;  # keypad keyboard control

# check is access code is correct, if so do something, like
# flip a relay for 3 seconds
def checkcode(code):
    if code=="123456":
        relay = YRelay.FirstRelay()
        if relay is not None: relay.pulse(3000);
        return True
    return False

# a key was pressed, lets update currentcode variable
def buttonPressed(value):
    global currentcode
    global refreshneeded
    print("key="+value);
    if value=='*': currentcode='' # the star key resets the current code
    elif (len(currentcode)<6) or (value=="#"): currentcode=currentcode+value;
    print("code="+currentcode)
    refreshneeded=True

# display refresh logic
def refresh(display,layer1,layer2):
    global currentcode
    global refreshneeded
    global keyboardDisabled
    refreshneeded=False
    layer1.clear()
    #no code entered yet
    if (currentcode==""):
      layer1.selectFont("Small.yfm")
      layer1.drawText(64, 16, YDisplayLayer.ALIGN.CENTER,"Enter code followed by #")
    # code + code : lets check it
    elif currentcode[-1:]=="#":
      layer1.selectFont("Medium.yfm")
      if checkcode(currentcode[:-1]):  # is code correct ?
        layer1.drawText(64, 16, YDisplayLayer.ALIGN.CENTER,"Access granted")
      else:
        layer1.drawText(64, 16, YDisplayLayer.ALIGN.CENTER,"Access denied")
      #display message and disable keyboard oad for 3 seconds
      display.swapLayerContent(1, 2)
      keyboardDisabled = True;
      YAPI.Sleep(3000, errmsg)   # wait for 2 secs
      keyboardDisabled = False;
      # reset
      currentcode="";
      refreshneeded=True;
      layer2.clear();
      layer1.clear();
    # display the current code being entered. In a real life application
    # displaying the actual code might no such a good idea.
    else:
      layer1.selectFont("Large.yfm")
      layer1.drawText(64, 16, YDisplayLayer.ALIGN.CENTER,currentcode)
    display.swapLayerContent(1, 2)

def anButtonChange(fct,value):

  global lastfunction
  global lastvalue
  global keyboardDisabled
  table ="#0*987654321"

  if (keyboardDisabled): return;

  fctname = str(fct)
  v = int(value)
  # button index is row
  buttonindex= int(fctname[-1:])-1

  if fctname == lastfunction:
    if (v >900) : #keypad key went back to normal position
      if (lastvalue <200) : i =0  #minimal value is column
      elif lastvalue<330  : i =1
      elif lastvalue<500  : i =2
      # compute keypad key index in the table
      index = buttonindex*3 +i
      # a key was pressed, lets notify it
      buttonPressed(table[index:index+1])
      # reset
      lastfunction  =""
      lastvalue   =0
    else: # memorize minimal value
        if v<lastvalue: lastvalue=v;
  else:
    if v<500: #memorize function name and value
       lastfunction = fctname
       lastvalue = v

# Setup the API
errmsg=YRefParam()
if YAPI.RegisterHub("usb", errmsg)!= YAPI.SUCCESS:
    sys.exit("local USB init error"+errmsg.value)

# If you are using an ethernet version, uncomment the
# two following lines and set the proper IP address

#if YAPI.RegisterHub("172.17.17.96", errmsg)!= YAPI.SUCCESS:
#    sys.exit("YoctoHub-Ethernet init error"+errmsg.value)

# find a configure the display
display= YDisplay.FirstDisplay()
if display is  None: sys.exit("No display found")
serial=display.get_module().get_serialNumber()
print("using "+serial+" device")
display.resetAll()
layer1 = display.get_displayLayer(1) #we will use double buffering
layer2 = display.get_displayLayer(2)
layer1.hide()

# find and configure the  buttons
for i in range(5):
  b = YAnButton.FindAnButton(serial+'.anButton'+str(i+1))
  if not(b.isOnline()):   sys.exit("anbutton"+str(i+1)+" found")
  # reset calibration, just in case
  b.set_calibrationMax(4095)
  b.set_calibrationMin(0)
  # no need to be too sensitive
  b.set_sensitivity(550)
  # when a button value change, the anButtonChange  function will be called
  b.registerValueCallback(anButtonChange)

while True:
    YAPI.Sleep(50, errmsg)   # traps callback events
    if (refreshneeded): refresh(display,layer1,layer2)












